package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedHashSet;
import org.openxma.dsl.reference.base.ContactInfo;
import org.openxma.dsl.reference.dao.CustomerDao;
import org.openxma.dsl.reference.dto.CustomerAddressDto;
import org.openxma.dsl.reference.dto.CustomerContactReport;
import org.openxma.dsl.reference.dto.CustomerIdDto;
import org.openxma.dsl.reference.dto.CustomerOrderReportDto;
import org.openxma.dsl.reference.dto.CustomerView;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.impl.CustomerImpl;
import org.openxma.dsl.reference.types.Gender;
import org.openxma.dsl.reference.types.valueobject.OrderNumber;
/**
 * @see CustomerDao
 */
@SuppressWarnings("unused")
public abstract class CustomerDaoGenImpl extends GenericDaoHibernateImpl<Customer,String> implements CustomerDao {
	
	public CustomerDaoGenImpl() {
		super(CustomerImpl.class);
	}
	
	/**
	 * @param context the context for the object creation
	 * @return a transient instance of entity <code>Customer</code> with an already assigned identifier.
	 */
	public Customer createEntity(Object context) {
		return create();
	}
	
	public Customer create() 
	{
		Customer customer = createEntityInstance();
		customer.setOid(dataFieldMaxValueIncrementer.nextStringValue());
		return customer;
	}
	
	public Customer create(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate) {
		Assert.notNull(invoiceAddress, "Parameter 'invoiceAddress' must not be null");
		Assert.notNull(deliveryAddress, "Parameter 'deliveryAddress' must not be null");
		Assert.notNull(customerFirstName, "Parameter 'customerFirstName' must not be null");
		Assert.notNull(customerLastName, "Parameter 'customerLastName' must not be null");
		Assert.notNull(gender, "Parameter 'gender' must not be null");
		Assert.notNull(emailAddress, "Parameter 'emailAddress' must not be null");
		Assert.notNull(customerBirthDate, "Parameter 'customerBirthDate' must not be null");
		Customer customer = (Customer) create();
		customer.setInvoiceAddress(invoiceAddress);
		customer.setDeliveryAddress(deliveryAddress);
		customer.setFirstName(customerFirstName);
		customer.setLastName(customerLastName);
		customer.setGender(gender);
		customer.setEmailAddress(emailAddress);
		customer.setBirthDate(customerBirthDate);
		return customer;
	}
	
	public Customer createAndSave(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate) {
		Customer customer = create(invoiceAddress,deliveryAddress,customerFirstName,customerLastName,gender,emailAddress,customerBirthDate);
		saveOrUpdate(customer);
		return customer;
	}
	
	public Customer loadByCustomerNk(String customerFirstName,String customerLastName,Date customerBirthDate) {
		Criterion criterion = equal("firstName", customerFirstName, false);
		criterion = and(criterion,"lastName", customerLastName, false);
		criterion = and(criterion,"birthDate", customerBirthDate, false);
		Criteria criteria = createCriteria(criterion);
		return (Customer) unique(criteria);
	}
	
	public Customer loadByCustomerUkEmail(String emailAddress) {
		Criterion criterion = equal("emailAddress", emailAddress, false);
		Criteria criteria = createCriteria(criterion);
		return (Customer) unique(criteria);
	}
	
	public Collection<CustomerOrderReportDto> testSubQueryBug() {
		String queryName = CustomerDao.TEST_SUB_QUERY_BUG;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyTestSubQueryBugQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerOrderReportDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyTestSubQueryBugQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerOrderReportDto mapTestSubQueryBugTuple(Object[] tuple, int rowNumber) {
		CustomerOrderReportDto customerOrderReportDto = null;
		if (tuple!=null) {
			customerOrderReportDto = new CustomerOrderReportDto();
		}
		return customerOrderReportDto;
	}
	
	public Collection<CustomerOrderReportDto> testSubQueryBugWithStaticWhereClause() {
		String queryName = CustomerDao.TEST_SUB_QUERY_BUG_WITH_STATIC_WHERE_CLAUSE;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyTestSubQueryBugWithStaticWhereClauseQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerOrderReportDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyTestSubQueryBugWithStaticWhereClauseQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerOrderReportDto mapTestSubQueryBugWithStaticWhereClauseTuple(Object[] tuple, int rowNumber) {
		CustomerOrderReportDto customerOrderReportDto = null;
		if (tuple!=null) {
			customerOrderReportDto = new CustomerOrderReportDto();
		}
		return customerOrderReportDto;
	}
	
	public Collection<CustomerAddressDto> testSubSelectBug() {
		String queryName = CustomerDao.TEST_SUB_SELECT_BUG;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyTestSubSelectBugQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerAddressDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyTestSubSelectBugQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerAddressDto mapTestSubSelectBugTuple(Object[] tuple, int rowNumber) {
		CustomerAddressDto customerAddressDto = null;
		if (tuple!=null) {
			customerAddressDto = new CustomerAddressDto();
			customerAddressDto.setFirstName((String)tuple[0]);
			customerAddressDto.setLastName((String)tuple[1]);
			customerAddressDto.setEmailAddress((String)tuple[2]);
		}
		return customerAddressDto;
	}
	
	public Collection<CustomerAddressDto> testSubSelectBugInWhereClause() {
		String queryName = CustomerDao.TEST_SUB_SELECT_BUG_IN_WHERE_CLAUSE;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyTestSubSelectBugInWhereClauseQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerAddressDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyTestSubSelectBugInWhereClauseQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerAddressDto mapTestSubSelectBugInWhereClauseTuple(Object[] tuple, int rowNumber) {
		CustomerAddressDto customerAddressDto = null;
		if (tuple!=null) {
			customerAddressDto = new CustomerAddressDto();
			customerAddressDto.setFirstName((String)tuple[0]);
			customerAddressDto.setEmailAddress((String)tuple[1]);
		}
		return customerAddressDto;
	}
	
	public Collection<Customer> testCountDistinctBug() {
		String queryName = CustomerDao.TEST_COUNT_DISTINCT_BUG;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyTestCountDistinctBugQueryHints(namedQuery);
		return list(namedQuery);
	}
	
	protected void applyTestCountDistinctBugQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<CustomerOrderReportDto> generateCustomerOrderReport() {
		String queryName = CustomerDao.GENERATE_CUSTOMER_ORDER_REPORT;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyGenerateCustomerOrderReportQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerOrderReportDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyGenerateCustomerOrderReportQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerOrderReportDto mapGenerateCustomerOrderReportTuple(Object[] tuple, int rowNumber) {
		CustomerOrderReportDto customerOrderReportDto = null;
		if (tuple!=null) {
			customerOrderReportDto = new CustomerOrderReportDto();
			customerOrderReportDto.setFirstName((String)tuple[0]);
			customerOrderReportDto.setLastName((String)tuple[1]);
			customerOrderReportDto.setEmailAddress((String)tuple[2]);
			customerOrderReportDto.setCity((String)tuple[3]);
			customerOrderReportDto.setStreetName((String)tuple[4]);
			customerOrderReportDto.setStreetNumber((String)tuple[5]);
			customerOrderReportDto.setBillingCountry((String)tuple[6]);
			customerOrderReportDto.setDeliveryCountry((String)tuple[7]);
			customerOrderReportDto.setOrderNumber((OrderNumber)tuple[8]);
			customerOrderReportDto.setOrderState((String)tuple[9]);
			customerOrderReportDto.setTotalAmount((Double)tuple[10]);
			customerOrderReportDto.setQuantity((Long)tuple[11]);
			customerOrderReportDto.setName((String)tuple[12]);
			customerOrderReportDto.setFileName((String)tuple[13]);
		}
		return customerOrderReportDto;
	}
	
	public Collection<CustomerAddressDto> loadCustomerAddressDto() {
		String queryName = CustomerDao.LOAD_CUSTOMER_ADDRESS_DTO;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyLoadCustomerAddressDtoQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerAddressDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyLoadCustomerAddressDtoQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerAddressDto mapLoadCustomerAddressDtoTuple(Object[] tuple, int rowNumber) {
		CustomerAddressDto customerAddressDto = null;
		if (tuple!=null) {
			customerAddressDto = new CustomerAddressDto();
			customerAddressDto.setFirstName((String)tuple[0]);
			customerAddressDto.setLastName((String)tuple[1]);
			customerAddressDto.setEmailAddress((String)tuple[2]);
			customerAddressDto.setCity((String)tuple[3]);
		}
		return customerAddressDto;
	}
	
	public Collection<CustomerAddressDto> loadCustomerAddressDtoJoinBillingAddress() {
		String queryName = CustomerDao.LOAD_CUSTOMER_ADDRESS_DTO_JOIN_BILLING_ADDRESS;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyLoadCustomerAddressDtoJoinBillingAddressQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerAddressDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyLoadCustomerAddressDtoJoinBillingAddressQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerAddressDto mapLoadCustomerAddressDtoJoinBillingAddressTuple(Object[] tuple, int rowNumber) {
		CustomerAddressDto customerAddressDto = null;
		if (tuple!=null) {
			customerAddressDto = new CustomerAddressDto();
			customerAddressDto.setFirstName((String)tuple[0]);
			customerAddressDto.setLastName((String)tuple[1]);
			customerAddressDto.setEmailAddress((String)tuple[2]);
			customerAddressDto.setBillingAddressCity((String)tuple[3]);
			customerAddressDto.setCity((String)tuple[4]);
		}
		return customerAddressDto;
	}
	
	public Collection<Customer> findAllByFirstNameAndLastName(String firstName,String lastName) {
		String queryName = CustomerDao.FIND_ALL_BY_FIRST_NAME_AND_LAST_NAME0;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("firstName", firstName);
		namedQuery.setParameter("lastName", lastName);
		applyFindAllByFirstNameAndLastNameQueryHints(namedQuery,firstName,lastName);
		return list(namedQuery);
	}
	
	protected void applyFindAllByFirstNameAndLastNameQueryHints(Query query,String firstName,String lastName) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findAllByFirstNameAndLastName(String firstName,String lastName,String emailAddress) {
		String queryName = CustomerDao.FIND_ALL_BY_FIRST_NAME_AND_LAST_NAME1;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("firstName", firstName);
		namedQuery.setParameter("lastName", lastName);
		namedQuery.setParameter("emailAddress", emailAddress);
		applyFindAllByFirstNameAndLastNameQueryHints(namedQuery,firstName,lastName,emailAddress);
		return list(namedQuery);
	}
	
	protected void applyFindAllByFirstNameAndLastNameQueryHints(Query query,String firstName,String lastName,String emailAddress) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findAllLikeNameJoinOrders(String firstName,String lastName) {
		String queryName = CustomerDao.FIND_ALL_LIKE_NAME_JOIN_ORDERS;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("firstName", firstName);
		namedQuery.setParameter("lastName", lastName);
		applyFindAllLikeNameJoinOrdersQueryHints(namedQuery,firstName,lastName);
		Collection<Customer> result = list(namedQuery);
		return new LinkedHashSet<Customer>(result);
	}
	
	protected void applyFindAllLikeNameJoinOrdersQueryHints(Query query,String firstName,String lastName) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findByNamedQueryCustomerLastName(String lastName) {
		String queryName = CustomerDao.FIND_BY_NAMED_QUERY_CUSTOMER_LAST_NAME;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("lastName", lastName);
		applyFindByNamedQueryCustomerLastNameQueryHints(namedQuery,lastName);
		return list(namedQuery);
	}
	
	protected void applyFindByNamedQueryCustomerLastNameQueryHints(Query query,String lastName) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findAllLikeLastNamePageable(String lastName,Integer firstResult,Integer maxResults) {
		String queryName = CustomerDao.FIND_ALL_LIKE_LAST_NAME_PAGEABLE;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("lastName", lastName);
		applyFindAllLikeLastNamePageableQueryHints(namedQuery,lastName,firstResult,maxResults);
		return list(namedQuery);
	}
	
	protected void applyFindAllLikeLastNamePageableQueryHints(Query query,String lastName,Integer firstResult,Integer maxResults) {
		if (firstResult!=null) {
			query.setFirstResult(firstResult);
		}
		if (maxResults!=null) {
			query.setMaxResults(maxResults);
		}
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findAllInName(Collection<String> lastName,Integer firstResult,Integer maxResults) {
		String queryName = CustomerDao.FIND_ALL_IN_NAME;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameterList("lastName", lastName);
		applyFindAllInNameQueryHints(namedQuery,lastName,firstResult,maxResults);
		return list(namedQuery);
	}
	
	protected void applyFindAllInNameQueryHints(Query query,Collection<String> lastName,Integer firstResult,Integer maxResults) {
		if (firstResult!=null) {
			query.setFirstResult(firstResult);
		}
		if (maxResults!=null) {
			query.setMaxResults(maxResults);
		}
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findByBirthDate(Date birthDate) {
		String queryName = CustomerDao.FIND_BY_BIRTH_DATE;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setDate("birthDate", birthDate);
		applyFindByBirthDateQueryHints(namedQuery,birthDate);
		return list(namedQuery);
	}
	
	protected void applyFindByBirthDateQueryHints(Query query,Date birthDate) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<CustomerContactReport> loadCustomerContactReport() {
		String queryName = CustomerDao.LOAD_CUSTOMER_CONTACT_REPORT;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyLoadCustomerContactReportQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerContactReport.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyLoadCustomerContactReportQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerContactReport mapLoadCustomerContactReportTuple(Object[] tuple, int rowNumber) {
		CustomerContactReport customerContactReport = null;
		if (tuple!=null) {
			customerContactReport = new CustomerContactReport();
			customerContactReport.setFirstName((String)tuple[0]);
			customerContactReport.setLastName((String)tuple[1]);
			customerContactReport.setEmailAddress((String)tuple[2]);
		}
		return customerContactReport;
	}
	
	public CustomerContactReport loadUniqueCustomerContactReport(String lastName) {
		String queryName = CustomerDao.LOAD_UNIQUE_CUSTOMER_CONTACT_REPORT;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("lastName", lastName);
		applyLoadUniqueCustomerContactReportQueryHints(namedQuery,lastName);
		Object[] tuple = unique(namedQuery);
		return mapLoadUniqueCustomerContactReportTuple(tuple);
	}
	
	protected void applyLoadUniqueCustomerContactReportQueryHints(Query query,String lastName) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerContactReport mapLoadUniqueCustomerContactReportTuple(Object[] tuple) {
		CustomerContactReport customerContactReport = null;
		if (tuple!=null) {
			customerContactReport = new CustomerContactReport();
			customerContactReport.setFirstName((String)tuple[0]);
			customerContactReport.setLastName((String)tuple[1]);
			customerContactReport.setEmailAddress((String)tuple[2]);
		}
		return customerContactReport;
	}
	
	public Collection<CustomerIdDto> findAllCustomerIds() {
		String queryName = CustomerDao.FIND_ALL_CUSTOMER_IDS;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyFindAllCustomerIdsQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerIdDto.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyFindAllCustomerIdsQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerIdDto mapFindAllCustomerIdsTuple(Object[] tuple, int rowNumber) {
		CustomerIdDto customerIdDto = null;
		if (tuple!=null) {
			customerIdDto = new CustomerIdDto();
			customerIdDto.setOid((String)tuple[0]);
		}
		return customerIdDto;
	}
	
	public Collection<String> findAllCustomerIdLikeName(String name) {
		String queryName = CustomerDao.FIND_ALL_CUSTOMER_ID_LIKE_NAME;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("name", name);
		applyFindAllCustomerIdLikeNameQueryHints(namedQuery,name);
		return list(namedQuery);
	}
	
	protected void applyFindAllCustomerIdLikeNameQueryHints(Query query,String name) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findCustomerByContact(ContactInfo contactInfo) {
		String queryName = CustomerDao.FIND_CUSTOMER_BY_CONTACT;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("contactInfo_email", contactInfo.getEmail());
		applyFindCustomerByContactQueryHints(namedQuery,contactInfo);
		return list(namedQuery);
	}
	
	protected void applyFindCustomerByContactQueryHints(Query query,ContactInfo contactInfo) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findByCustomerView(CustomerView customerView) {
		String queryName = CustomerDao.FIND_BY_CUSTOMER_VIEW;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("customerView_emailAddress", customerView.getEmailAddress());
		applyFindByCustomerViewQueryHints(namedQuery,customerView);
		return list(namedQuery);
	}
	
	protected void applyFindByCustomerViewQueryHints(Query query,CustomerView customerView) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findCustomerByExample(Customer customer) {
		String queryName = CustomerDao.FIND_CUSTOMER_BY_EXAMPLE;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("customer_emailAddress", customer.getEmailAddress());
		namedQuery.setParameter("customer_firstName", customer.getFirstName());
		namedQuery.setParameter("customer_lastName", customer.getLastName());
		applyFindCustomerByExampleQueryHints(namedQuery,customer);
		return list(namedQuery);
	}
	
	protected void applyFindCustomerByExampleQueryHints(Query query,Customer customer) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public void deleteCustomerByName(String lastName) {
		String queryName = CustomerDao.DELETE_CUSTOMER_BY_NAME;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("lastName", lastName);
		applyDeleteCustomerByNameQueryHints(namedQuery,lastName);
		namedQuery.executeUpdate();
	}
	
	protected void applyDeleteCustomerByNameQueryHints(Query query,String lastName) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public void updateCustomer(String lastName,String newLastName) {
		String queryName = CustomerDao.UPDATE_CUSTOMER;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("lastName", lastName);
		namedQuery.setParameter("newLastName", newLastName);
		applyUpdateCustomerQueryHints(namedQuery,lastName,newLastName);
		namedQuery.executeUpdate();
	}
	
	protected void applyUpdateCustomerQueryHints(Query query,String lastName,String newLastName) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected Mapper getNamedQueryMapper(String queryName) {
		return new CustomerDaoQueryMapper();
	}
	
	protected class CustomerDaoQueryMapper extends NamedQueryMapper {
		
	@SuppressWarnings("unchecked")
	public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
		if ("Customer.TestSubQueryBug".equals(getQueryName(context))) {
			return (E) mapTestSubQueryBugTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.TestSubQueryBugWithStaticWhereClause".equals(getQueryName(context))) {
			return (E) mapTestSubQueryBugWithStaticWhereClauseTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.TestSubSelectBug".equals(getQueryName(context))) {
			return (E) mapTestSubSelectBugTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.TestSubSelectBugInWhereClause".equals(getQueryName(context))) {
			return (E) mapTestSubSelectBugInWhereClauseTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.GenerateCustomerOrderReport".equals(getQueryName(context))) {
			return (E) mapGenerateCustomerOrderReportTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.LoadCustomerAddressDto".equals(getQueryName(context))) {
			return (E) mapLoadCustomerAddressDtoTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.LoadCustomerAddressDtoJoinBillingAddress".equals(getQueryName(context))) {
			return (E) mapLoadCustomerAddressDtoJoinBillingAddressTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.LoadCustomerContactReport".equals(getQueryName(context))) {
			return (E) mapLoadCustomerContactReportTuple((Object[]) sourceObject, 1);
		}
		if ("Customer.FindAllCustomerIds".equals(getQueryName(context))) {
			return (E) mapFindAllCustomerIdsTuple((Object[]) sourceObject, 1);
		}
		return super.mapOne(sourceObject, targetObject);
	}
}
}
