package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Collection;
import org.openxma.dsl.reference.dao.ProductImageDao;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.ProductImage;
import org.openxma.dsl.reference.model.impl.ProductImageImpl;
/**
 * @see ProductImageDao
 */
@SuppressWarnings("unused")
public abstract class ProductImageDaoGenImpl extends GenericDaoHibernateImpl<ProductImage,String> implements ProductImageDao {
	
	public ProductImageDaoGenImpl() {
		super(ProductImageImpl.class);
	}
	
	/**
	 * @param context the context for the object creation
	 * @return a transient instance of entity <code>ProductImage</code> with an already assigned identifier.
	 */
	public ProductImage createEntity(Object context) {
		return create();
	}
	
	public ProductImage create() 
	{
		ProductImage productImage = createEntityInstance();
		productImage.setOid(dataFieldMaxValueIncrementer.nextStringValue());
		return productImage;
	}
	
	public ProductImage create(Product product,String name,String fileName) {
		Assert.notNull(product, "Parameter 'product' must not be null");
		Assert.notNull(name, "Parameter 'name' must not be null");
		Assert.notNull(fileName, "Parameter 'fileName' must not be null");
		ProductImage productImage = (ProductImage) create();
		productImage.setProduct(product);
		productImage.setName(name);
		productImage.setFileName(fileName);
		return productImage;
	}
	
	public ProductImage createAndSave(Product product,String name,String fileName) {
		ProductImage productImage = create(product,name,fileName);
		saveOrUpdate(productImage);
		return productImage;
	}
	
	public Collection<ProductImage> findAllByMatchingSize(Collection<Integer> sizeX,Collection<Integer> sizeY) {
		String queryName = ProductImageDao.FIND_ALL_BY_MATCHING_SIZE;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameterList("sizeX", sizeX);
		namedQuery.setParameterList("sizeY", sizeY);
		applyFindAllByMatchingSizeQueryHints(namedQuery,sizeX,sizeY);
		return list(namedQuery);
	}
	
	protected void applyFindAllByMatchingSizeQueryHints(Query query,Collection<Integer> sizeX,Collection<Integer> sizeY) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected Mapper getNamedQueryMapper(String queryName) {
		return new ProductImageDaoQueryMapper();
	}
	
	protected class ProductImageDaoQueryMapper extends NamedQueryMapper {
		
	@SuppressWarnings("unchecked")
	public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
		return super.mapOne(sourceObject, targetObject);
	}
}
}
