package org.openxma.dsl.reference.jsr303;

import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.List;
import org.openxma.dsl.platform.assembler.DtoAssembler;
import org.openxma.dsl.platform.assembler.DtoAssemblerFactory;
import org.openxma.dsl.platform.assembler.DtoAssemblerRegistry;
import org.openxma.dsl.platform.converter.Converter;
import org.openxma.dsl.reference.jsr303.dto.Jsr303BaseEntityView;
import org.openxma.dsl.reference.jsr303.dto.Jsr303TestEntityView;
import org.openxma.dsl.reference.jsr303.model.Jsr303BaseEntity;
import org.openxma.dsl.reference.jsr303.model.Jsr303TestEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;

public class DtoAssemblerFactoryJsr303Gen implements DtoAssemblerFactory {

	private List<DtoAssembler<?>> dtoAssemblers;

	@Autowired
	private DtoAssemblerRegistry dtoAssemblerRegistry;
	
	@Autowired
	private Converter converter;

	public DtoAssemblerFactoryJsr303Gen() {
		dtoAssemblers = new ArrayList<DtoAssembler<?>>(2);
		initDtoAssemblers();
	}


	public List<DtoAssembler<?>> getDtoAssemblers() {
		return dtoAssemblers;
	}

	protected DtoAssemblerRegistry getDtoAssemblerRegistry() {
		return dtoAssemblerRegistry;
	}
	
	protected Converter getConverter() {
		return converter;
	}

	private void initDtoAssemblers() {
		dtoAssemblers.add(createDtoAssemblerJsr303BaseEntityView());
		dtoAssemblers.add(createDtoAssemblerJsr303TestEntityView());
	}


	private DtoAssembler<Jsr303BaseEntityView> createDtoAssemblerJsr303BaseEntityView() {
		return new DtoAssembler<Jsr303BaseEntityView>() {
			public Jsr303BaseEntityView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				if (sourceEntity instanceof Jsr303BaseEntity) {
					Jsr303BaseEntity jsr303BaseEntity = (Jsr303BaseEntity) sourceEntity;
					Jsr303BaseEntityView result = mapFromJsr303BaseEntityToJsr303BaseEntityView(jsr303BaseEntity);
					return result;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public Jsr303BaseEntityView mapFromEntity(Object sourceEntity, Jsr303BaseEntityView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
				if (sourceEntity instanceof Jsr303BaseEntity) {
					Jsr303BaseEntity jsr303BaseEntity = (Jsr303BaseEntity) sourceEntity;
					mapFromJsr303BaseEntityToJsr303BaseEntityView(jsr303BaseEntity, targetDto);
					return targetDto;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public <T> T mapTo(Jsr303BaseEntityView sourceDto, Class<T> target) {
				if (Jsr303BaseEntityView.class.equals(target)) {
					Jsr303BaseEntityView result = new Jsr303BaseEntityView();
					mapFromJsr303BaseEntityViewToJsr303BaseEntityView(sourceDto, result);
					return target.cast(result);
				}
				else if (Jsr303BaseEntity.class.equals(target)) {
					return target.cast(mapFromJsr303BaseEntityViewToJsr303BaseEntity(sourceDto));
				}
				else {
					throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
				}
			}
			
			public void mapTo(Jsr303BaseEntityView sourceDto, Object targetObject) {
				if (targetObject instanceof Jsr303BaseEntityView) {
					mapFromJsr303BaseEntityViewToJsr303BaseEntityView(sourceDto, (Jsr303BaseEntityView) targetObject);
				}
				else if (targetObject instanceof Jsr303BaseEntity) {
					mapFromJsr303BaseEntityViewToJsr303BaseEntity(sourceDto, (Jsr303BaseEntity)targetObject);
				}			   
				else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}
	
	protected void mapFromJsr303BaseEntityViewToJsr303BaseEntityView(Jsr303BaseEntityView sourceDto, Jsr303BaseEntityView jsr303BaseEntityView) {
		jsr303BaseEntityView.setBaseString(sourceDto.getBaseString());
	}
	
	protected Jsr303BaseEntity mapFromJsr303BaseEntityViewToJsr303BaseEntity(Jsr303BaseEntityView sourceDto) {
		return mapFromJsr303BaseEntityViewToJsr303BaseEntity(sourceDto, MapperContextHolder.createEntity(Jsr303BaseEntity.class));
	}
	
	protected Jsr303BaseEntity mapFromJsr303BaseEntityViewToJsr303BaseEntity(Jsr303BaseEntityView sourceDto, Jsr303BaseEntity jsr303BaseEntity) {
		if (MapperContextHolder.getContext().containsKey(sourceDto)) {
			return (Jsr303BaseEntity) MapperContextHolder.getContext().get(sourceDto);
		}
		MapperContextHolder.getContext().put(sourceDto, jsr303BaseEntity);
		jsr303BaseEntity.setBaseString(sourceDto.getBaseString());
		return jsr303BaseEntity;
	}
	
	protected Jsr303BaseEntityView mapFromJsr303BaseEntityToJsr303BaseEntityView(Jsr303BaseEntity jsr303BaseEntity) {
		if (MapperContextHolder.getContext().containsKey(jsr303BaseEntity)) {
			return (Jsr303BaseEntityView) MapperContextHolder.getContext().get(jsr303BaseEntity);
		}
		Jsr303BaseEntityView jsr303BaseEntityView = new Jsr303BaseEntityView();
		MapperContextHolder.getContext().put(jsr303BaseEntity, jsr303BaseEntityView);
		jsr303BaseEntityView.setBaseString(jsr303BaseEntity.getBaseString());
		return jsr303BaseEntityView;
	}
	
	protected Jsr303BaseEntityView mapFromJsr303BaseEntityToJsr303BaseEntityView(Jsr303BaseEntity jsr303BaseEntity, Jsr303BaseEntityView jsr303BaseEntityView) {
		if (MapperContextHolder.getContext().containsKey(jsr303BaseEntity)) {
			return (Jsr303BaseEntityView) MapperContextHolder.getContext().get(jsr303BaseEntity);
		}
		MapperContextHolder.getContext().put(jsr303BaseEntity, jsr303BaseEntityView);
		jsr303BaseEntityView.setBaseString(jsr303BaseEntity.getBaseString());
		return jsr303BaseEntityView;
	}
	
	
	
	private DtoAssembler<Jsr303TestEntityView> createDtoAssemblerJsr303TestEntityView() {
		return new DtoAssembler<Jsr303TestEntityView>() {
			public Jsr303TestEntityView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				if (sourceEntity instanceof Jsr303TestEntity) {
					Jsr303TestEntity jsr303TestEntity = (Jsr303TestEntity) sourceEntity;
					Jsr303TestEntityView result = mapFromJsr303TestEntityToJsr303TestEntityView(jsr303TestEntity);
					return result;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public Jsr303TestEntityView mapFromEntity(Object sourceEntity, Jsr303TestEntityView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
				if (sourceEntity instanceof Jsr303TestEntity) {
					Jsr303TestEntity jsr303TestEntity = (Jsr303TestEntity) sourceEntity;
					mapFromJsr303TestEntityToJsr303TestEntityView(jsr303TestEntity, targetDto);
					return targetDto;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public <T> T mapTo(Jsr303TestEntityView sourceDto, Class<T> target) {
				if (Jsr303TestEntityView.class.equals(target)) {
					Jsr303TestEntityView result = new Jsr303TestEntityView();
					mapFromJsr303TestEntityViewToJsr303TestEntityView(sourceDto, result);
					return target.cast(result);
				}
				else if (Jsr303TestEntity.class.equals(target)) {
					return target.cast(mapFromJsr303TestEntityViewToJsr303TestEntity(sourceDto));
				}
				else {
					throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
				}
			}
			
			public void mapTo(Jsr303TestEntityView sourceDto, Object targetObject) {
				if (targetObject instanceof Jsr303TestEntityView) {
					mapFromJsr303TestEntityViewToJsr303TestEntityView(sourceDto, (Jsr303TestEntityView) targetObject);
				}
				else if (targetObject instanceof Jsr303TestEntity) {
					mapFromJsr303TestEntityViewToJsr303TestEntity(sourceDto, (Jsr303TestEntity)targetObject);
				}			   
				else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}
	
	protected void mapFromJsr303TestEntityViewToJsr303TestEntityView(Jsr303TestEntityView sourceDto, Jsr303TestEntityView jsr303TestEntityView) {
		jsr303TestEntityView.setBaseString(sourceDto.getBaseString());
		jsr303TestEntityView.setStr(sourceDto.getStr());
		jsr303TestEntityView.setCategory(sourceDto.getCategory());
	}
	
	protected Jsr303TestEntity mapFromJsr303TestEntityViewToJsr303TestEntity(Jsr303TestEntityView sourceDto) {
		return mapFromJsr303TestEntityViewToJsr303TestEntity(sourceDto, MapperContextHolder.createEntity(Jsr303TestEntity.class));
	}
	
	protected Jsr303TestEntity mapFromJsr303TestEntityViewToJsr303TestEntity(Jsr303TestEntityView sourceDto, Jsr303TestEntity jsr303TestEntity) {
		if (MapperContextHolder.getContext().containsKey(sourceDto)) {
			return (Jsr303TestEntity) MapperContextHolder.getContext().get(sourceDto);
		}
		MapperContextHolder.getContext().put(sourceDto, jsr303TestEntity);
		jsr303TestEntity.setBaseString(sourceDto.getBaseString());
		jsr303TestEntity.setStr(sourceDto.getStr());
		jsr303TestEntity.setCategory(sourceDto.getCategory());
		return jsr303TestEntity;
	}
	
	protected Jsr303TestEntityView mapFromJsr303TestEntityToJsr303TestEntityView(Jsr303TestEntity jsr303TestEntity) {
		if (MapperContextHolder.getContext().containsKey(jsr303TestEntity)) {
			return (Jsr303TestEntityView) MapperContextHolder.getContext().get(jsr303TestEntity);
		}
		Jsr303TestEntityView jsr303TestEntityView = new Jsr303TestEntityView();
		MapperContextHolder.getContext().put(jsr303TestEntity, jsr303TestEntityView);
		jsr303TestEntityView.setBaseString(jsr303TestEntity.getBaseString());
		jsr303TestEntityView.setStr(jsr303TestEntity.getStr());
		jsr303TestEntityView.setCategory(jsr303TestEntity.getCategory());
		return jsr303TestEntityView;
	}
	
	protected Jsr303TestEntityView mapFromJsr303TestEntityToJsr303TestEntityView(Jsr303TestEntity jsr303TestEntity, Jsr303TestEntityView jsr303TestEntityView) {
		if (MapperContextHolder.getContext().containsKey(jsr303TestEntity)) {
			return (Jsr303TestEntityView) MapperContextHolder.getContext().get(jsr303TestEntity);
		}
		MapperContextHolder.getContext().put(jsr303TestEntity, jsr303TestEntityView);
		jsr303TestEntityView.setBaseString(jsr303TestEntity.getBaseString());
		jsr303TestEntityView.setStr(jsr303TestEntity.getStr());
		jsr303TestEntityView.setCategory(jsr303TestEntity.getCategory());
		return jsr303TestEntityView;
	}
	
	
	
}
