package org.openxma.dsl.reference.model.impl;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import org.openxma.dsl.reference.model.Category;
import org.openxma.dsl.reference.model.Product;

public abstract class CategoryGenImpl implements Category {
	protected Long oid;
	protected String name;
	protected String displayName;
	protected Category parentCategory;
	protected Set<Category> subCategories;
	protected Set<Product> products;
	
	public Long getOid() {
		return this.oid;
	}
	
	public void setOid(Long oid) {
		this.oid = oid;
	}
	
	public String getName() {
		return this.name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public String getDisplayName() {
		return this.displayName;
	}
	
	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}
	
	public Category getParentCategory() {
		return this.parentCategory;
	}
	
	public void setParentCategory(Category parentCategory) {
		if (this.parentCategory != parentCategory) {
			if (this.parentCategory != null) {
				this.parentCategory.removeSubCategories(this);
			}
			this.parentCategory = parentCategory;
			if (parentCategory != null) {
				parentCategory.addSubCategories(this);
			}
		}
	}
	
	public Set<Category> getSubCategories() {
		if (this.subCategories == null) {
			this.subCategories = new HashSet<Category>();
		}
		return Collections.unmodifiableSet(this.subCategories);
	}
	
	public boolean hasSubCategories() {
		return subCategories != null && !subCategories.isEmpty();
	}
	
	public void addSubCategories(Category subCategories) {
		if (subCategories == null) {
			throw new IllegalArgumentException("parameter 'subCategories' must not be null");
		}
		if (!getSubCategories().contains(subCategories)) {
			this.subCategories.add(subCategories);
			subCategories.setParentCategory(this);
		}
	}
	
	public void removeSubCategories(Category subCategories) {
		if (subCategories == null) {
			throw new IllegalArgumentException("parameter 'subCategories' must not be null");
		}
		if (getSubCategories().contains(subCategories)) {
			this.subCategories.remove(subCategories);
			subCategories.setParentCategory(null);
		}
	}
	
	public void removeAllSubCategories() {
		if (this.subCategories != null) {
			for (Category category : new HashSet<Category>(this.subCategories)) {
				removeSubCategories(category);
			}
		}
	}
	
	
	public Set<Product> getProducts() {
		if (this.products == null) {
			this.products = new HashSet<Product>();
		}
		return Collections.unmodifiableSet(this.products);
	}
	
	public boolean hasProducts() {
		return products != null && !products.isEmpty();
	}
	
	public void addProducts(Product products) {
		if (products == null) {
			throw new IllegalArgumentException("parameter 'products' must not be null");
		}
		if (!getProducts().contains(products)) {
			this.products.add(products);
			products.addCategories(this);
		}
	}
	
	public void removeProducts(Product products) {
		if (products == null) {
			throw new IllegalArgumentException("parameter 'products' must not be null");
		}
		if (getProducts().contains(products)) {
			this.products.remove(products);
			products.removeCategories(this);
		}
	}
	
	public void removeAllProducts() {
		if (this.products != null) {
			for (Product product : new HashSet<Product>(this.products)) {
				removeProducts(product);
			}
		}
	}
	

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((getOid() == null) ? 0 : getOid().hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Category)) {
			return false;
		}
		Category other = (Category) obj;
		if (getOid() == null) {
			if (other.getOid() != null) {
				return false;
			}
		} else if (!getOid().equals(other.getOid())) {
			return false;
		}
		return true;
	}
	
	@Override
	public String toString() {
		StringBuilder toStringBuilder = new StringBuilder("Category [");
		toStringBuilder.append("oid=").append(getOid()).append(",");
		toStringBuilder.append("name=").append(getName()).append(",");
		toStringBuilder.append("displayName=").append(getDisplayName());
		return toStringBuilder.append("]").toString();
	}
}
