package org.openxma.dsl.reference.model.impl;

import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import org.openxma.dsl.reference.model.Category;
import org.openxma.dsl.reference.model.Certificate;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.ProductImage;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.types.valueobject.ProductId;

public abstract class ProductGenImpl implements Product {
	protected ProductId oid;
	protected Date version;
	protected String name;
	protected Integer unitPrice;
	protected Integer unitOnStock;
	protected Integer unitOnOrder;
	protected Supplier supplier;
	protected ProductImage productImage;
	protected Set<Category> categories;
	protected Certificate cert1;
	protected Certificate cert2;
	protected Certificate cert3;
	
	public ProductId getOid() {
		return this.oid;
	}
	
	public void setOid(ProductId oid) {
		this.oid = oid;
	}
	
	public Date getVersion() {
		return this.version;
	}
	
	public void setVersion(Date version) {
		this.version = version;
	}
	
	public String getName() {
		return this.name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public Integer getUnitPrice() {
		return this.unitPrice;
	}
	
	public void setUnitPrice(Integer unitPrice) {
		this.unitPrice = unitPrice;
	}
	
	public Integer getUnitOnStock() {
		return this.unitOnStock;
	}
	
	public void setUnitOnStock(Integer unitOnStock) {
		this.unitOnStock = unitOnStock;
	}
	
	public Integer getUnitOnOrder() {
		return this.unitOnOrder;
	}
	
	public void setUnitOnOrder(Integer unitOnOrder) {
		this.unitOnOrder = unitOnOrder;
	}
	
	public Supplier getSupplier() {
		return this.supplier;
	}
	
	public void setSupplier(Supplier supplier) {
		if (this.supplier != supplier) {
			this.supplier = supplier;
		}
	}
	
	public ProductImage getProductImage() {
		return this.productImage;
	}
	
	public void setProductImage(ProductImage productImage) {
		if (this.productImage != productImage) {
			this.productImage = productImage;
			if (productImage != null) {
				productImage.setProduct(this);
			}
		}
	}
	
	public Set<Category> getCategories() {
		if (this.categories == null) {
			this.categories = new HashSet<Category>();
		}
		return Collections.unmodifiableSet(this.categories);
	}
	
	public boolean hasCategories() {
		return categories != null && !categories.isEmpty();
	}
	
	public void addCategories(Category categories) {
		if (categories == null) {
			throw new IllegalArgumentException("parameter 'categories' must not be null");
		}
		if (!getCategories().contains(categories)) {
			this.categories.add(categories);
			categories.addProducts(this);
		}
	}
	
	public void removeCategories(Category categories) {
		if (categories == null) {
			throw new IllegalArgumentException("parameter 'categories' must not be null");
		}
		if (getCategories().contains(categories)) {
			this.categories.remove(categories);
			categories.removeProducts(this);
		}
	}
	
	public void removeAllCategories() {
		if (this.categories != null) {
			this.categories.clear();
		}
	}
	
	
	public Certificate getCert1() {
		return this.cert1;
	}
	
	public void setCert1(Certificate cert1) {
		if (this.cert1 != cert1) {
			this.cert1 = cert1;
			if (cert1 != null) {
				cert1.setProduct(this);
			}
		}
	}
	
	public Certificate getCert2() {
		return this.cert2;
	}
	
	public void setCert2(Certificate cert2) {
		if (this.cert2 != cert2) {
			this.cert2 = cert2;
			if (cert2 != null) {
				cert2.setProduct(this);
			}
		}
	}
	
	public Certificate getCert3() {
		return this.cert3;
	}
	
	public void setCert3(Certificate cert3) {
		if (this.cert3 != cert3) {
			this.cert3 = cert3;
			if (cert3 != null) {
				cert3.setProduct(this);
			}
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((getOid() == null) ? 0 : getOid().hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Product)) {
			return false;
		}
		Product other = (Product) obj;
		if (getOid() == null) {
			if (other.getOid() != null) {
				return false;
			}
		} else if (!getOid().equals(other.getOid())) {
			return false;
		}
		return true;
	}
	
	@Override
	public String toString() {
		StringBuilder toStringBuilder = new StringBuilder("Product [");
		toStringBuilder.append("oid=").append(getOid()).append(",");
		toStringBuilder.append("version=").append(getVersion()).append(",");
		toStringBuilder.append("name=").append(getName()).append(",");
		toStringBuilder.append("unitPrice=").append(getUnitPrice()).append(",");
		toStringBuilder.append("unitOnStock=").append(getUnitOnStock()).append(",");
		toStringBuilder.append("unitOnOrder=").append(getUnitOnOrder());
		return toStringBuilder.append("]").toString();
	}
}
