package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import java.util.Collection;
import java.util.Date;
import org.openxma.dsl.reference.dao.CustomerDao;
import org.openxma.dsl.reference.dao.OrderDao;
import org.openxma.dsl.reference.dao.OrderItemDao;
import org.openxma.dsl.reference.dao.ProductDao;
import org.openxma.dsl.reference.dto.CustomerView;
import org.openxma.dsl.reference.dto.OrderItemView;
import org.openxma.dsl.reference.dto.OrderStateView;
import org.openxma.dsl.reference.dto.OrderView;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.OrderItem;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.service.OrderService;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
public abstract class OrderServiceGenImpl implements OrderService {
	protected final Log logger = LogFactory.getLog(getClass());
	@Autowired(required=false)
	protected Mapper mapper;
	protected OrderDao orderDao;
	protected CustomerDao customerDao;
	protected OrderItemDao orderItemDao;
	protected ProductDao productDao;
	
	@Autowired
	public void setOrderDao(OrderDao orderDao) {
		this.orderDao = orderDao;
	}
	
	@Autowired
	public void setCustomerDao(CustomerDao customerDao) {
		this.customerDao = customerDao;
	}
	
	@Autowired
	public void setOrderItemDao(OrderItemDao orderItemDao) {
		this.orderItemDao = orderItemDao;
	}
	
	@Autowired
	public void setProductDao(ProductDao productDao) {
		this.productDao = productDao;
	}
	
	@Transactional
	public OrderView save(OrderView orderView) {
		Assert.notNull(orderView, "parameter 'orderView' must not be null");
		Order order = this.mapper.createAndMapOne(orderView, Order.class,"saveOrder");
		Customer customer = this.mapper.createAndMapOne(orderView, Customer.class,"customer");
		customer = customerDao.load(customer.getOid());
		order.setCustomer(customer);
		addOrderItems(order,orderView);
		this.orderDao.saveOrUpdate(order);
		return this.mapper.mapOne(order,new OrderView());
	}
	
	private void addOrderItems(Order parentOrder,OrderView orderView) {
		if (orderView.getOrderItems() != null) {
			for (OrderItemView _orderItemView : orderView.getOrderItems()) {
				OrderItem orderItem = this.mapper.createAndMapOne(_orderItemView, OrderItem.class,"saveOrderItem");
				Product product = this.mapper.createAndMapOne(_orderItemView, Product.class,"product");
				if (product.getOid() != null) {
					product = productDao.load(product.getOid());
					orderItem.setProduct(product);
				}
				parentOrder.addOrderItems(orderItem);
			}
		}
	}
	
	@Transactional(readOnly = true)
	public OrderView loadOrderView(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
		return this.mapper.createAndMapOne(this.orderDao.load(oid),OrderView.class);
	}
	
	@Transactional
	public void update(OrderView orderView) {
		Assert.notNull(orderView, "parameter 'orderView' must not be null");
		Order mappedOrder = this.mapper.createAndMapOne(orderView, Order.class);
		Order order = this.orderDao.load(mappedOrder.getOid());
		if (!order.getVersion().equals(mappedOrder.getVersion())) {
			throw new OptimisticLockingFailureException("Entity 'Order' was updated or deleted by another transaction.");
		}
		this.mapper.mapOne(orderView, order,"updateOrder");
	}
	
	@Transactional
	public void deleteOrder(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
		this.orderDao.delete(oid);
	}
	
	@Transactional
	public OrderView saveOrUpdate(OrderView orderView) {
		Assert.notNull(orderView, "parameter 'orderView' must not be null");
		if ( orderView.getVersion() == null ) {
			return this.save(orderView);
		}
		else {
			this.update(orderView);
			return orderView;
		}
	}
	
	
	@Transactional
	public Collection<OrderStateView> readOrderStateByCustomerAndDate(Long customerId,Date reportDate) {
		return this.orderDao.readOrderStateByCustomerAndDate(customerId,reportDate);
	}
	
	@Transactional
	public Collection<OrderView> findAllOrderStatesBetweenDate(OrderView order,Date from,Date to) {
		return this.mapper.createAndMapMany(this.orderDao.findAllOrderStatesBetweenDate(mapper.createAndMapOne(order,Order.class),from,to),OrderView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<OrderView> findAllByCustomer(CustomerView customer) {
		return this.mapper.createAndMapMany(this.orderDao.findAllByCustomer(mapper.createAndMapOne(customer,Customer.class)),OrderView.class);
	}
	
}
