package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import java.util.Collection;
import org.openxma.dsl.platform.query.Expression;
import org.openxma.dsl.platform.query.QueryObject;
import org.openxma.dsl.reference.SupplierAddress;
import org.openxma.dsl.reference.dao.CustomerDao;
import org.openxma.dsl.reference.dao.SupplierDao;
import org.openxma.dsl.reference.dto.SupplierNameView;
import org.openxma.dsl.reference.dto.SupplierNameView2;
import org.openxma.dsl.reference.dto.SupplierView;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.service.SupplierService;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
public abstract class SupplierServiceGenImpl implements SupplierService {
	protected final Log logger = LogFactory.getLog(getClass());
	@Autowired(required=false)
	protected Mapper mapper;
	protected SupplierDao supplierDao;
	protected CustomerDao customerDao;
	
	@Autowired
	public void setSupplierDao(SupplierDao supplierDao) {
		this.supplierDao = supplierDao;
	}
	
	@Autowired
	public void setCustomerDao(CustomerDao customerDao) {
		this.customerDao = customerDao;
	}
	
	@Transactional
	public SupplierView save(SupplierView supplierView) {
		Assert.notNull(supplierView, "parameter 'supplierView' must not be null");
		Supplier supplier = this.mapper.createAndMapOne(supplierView, Supplier.class,"saveSupplier");
		this.supplierDao.saveOrUpdate(supplier);
		return this.mapper.mapOne(supplier,new SupplierView());
	}
	
	@Transactional(readOnly = true)
	public SupplierView loadSupplierView(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
		return this.mapper.createAndMapOne(this.supplierDao.load(oid),SupplierView.class);
	}
	
	@Transactional
	public void update(SupplierView supplierView) {
		Assert.notNull(supplierView, "parameter 'supplierView' must not be null");
		Supplier mappedSupplier = this.mapper.createAndMapOne(supplierView, Supplier.class);
		Supplier supplier = this.supplierDao.load(mappedSupplier.getOid());
		if (!supplier.getVersion().equals(mappedSupplier.getVersion())) {
			throw new OptimisticLockingFailureException("Entity 'Supplier' was updated or deleted by another transaction.");
		}
		this.mapper.mapOne(supplierView, supplier,"updateSupplier");
	}
	
	@Transactional
	public void deleteSupplier(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
		this.supplierDao.delete(oid);
	}
	
	@Transactional
	public SupplierView saveOrUpdate(SupplierView supplierView) {
		Assert.notNull(supplierView, "parameter 'supplierView' must not be null");
		if ( supplierView.getVersion() == null ) {
			return this.save(supplierView);
		}
		else {
			this.update(supplierView);
			return supplierView;
		}
	}
	
	
	@Transactional(readOnly=true)
	public Collection<SupplierNameView> findAllByCompanyNamePaging(String companyName,Integer firstResult,Integer maxResults) {
		return this.supplierDao.findAllByCompanyNamePaging(companyName,firstResult,maxResults);
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierNameView> findAllByCompanyNamePaging(String companyName,Expression _filter) {
		QueryObject queryObject = QueryObject.createNamedQuery(SupplierDao.FIND_ALL_BY_COMPANY_NAME_PAGING, _filter);
		queryObject.withArgs("companyName", companyName);
		Collection<SupplierNameView> _result =  this.supplierDao.query(queryObject);
		return _result;
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierView> findAllByAddress(SupplierAddress address) {
		return this.mapper.createAndMapMany(this.supplierDao.findAllByAddress(address),SupplierView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierView> findAllByAddress(SupplierAddress address,Expression _filter) {
		QueryObject queryObject = QueryObject.createNamedQuery(SupplierDao.FIND_ALL_BY_ADDRESS, _filter);
		queryObject.withArgs("address_streetName", address.getStreetName());
		Collection<Supplier> _result =  this.supplierDao.query(queryObject);
		return this.mapper.createAndMapMany(_result,SupplierView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierView> findWithUnusedParameter(String companyName,String code) {
		return this.mapper.createAndMapMany(this.supplierDao.findWithUnusedParameter(companyName,code),SupplierView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierView> findWithUnusedParameter(String companyName,String code,Expression _filter) {
		QueryObject queryObject = QueryObject.createNamedQuery(SupplierDao.FIND_WITH_UNUSED_PARAMETER, _filter);
		queryObject.withArgs("companyName", companyName);
		Collection<Supplier> _result =  this.supplierDao.query(queryObject);
		return this.mapper.createAndMapMany(_result,SupplierView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierView> findProductSuppliers() {
		return this.mapper.createAndMapMany(this.supplierDao.findProductSuppliers(),SupplierView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierView> findProductSuppliers(Expression _filter) {
		QueryObject queryObject = QueryObject.createNamedQuery(SupplierDao.FIND_PRODUCT_SUPPLIERS, _filter);
		Collection<Supplier> _result =  this.supplierDao.query(queryObject);
		return this.mapper.createAndMapMany(_result,SupplierView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierNameView2> findAllSupplierNames() {
		return this.supplierDao.findAllSupplierNames();
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierNameView2> findAllSupplierNames(Expression _filter) {
		QueryObject queryObject = QueryObject.createNamedQuery(SupplierDao.FIND_ALL_SUPPLIER_NAMES, _filter);
		Collection<SupplierNameView2> _result =  this.supplierDao.query(queryObject);
		return _result;
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierNameView> findAllSupplierNames2() {
		return this.supplierDao.findAllSupplierNames2();
	}
	
	@Transactional(readOnly=true)
	public Collection<SupplierNameView> findAllSupplierNames2(Expression _filter) {
		QueryObject queryObject = QueryObject.createNamedQuery(SupplierDao.FIND_ALL_SUPPLIER_NAMES2, _filter);
		Collection<SupplierNameView> _result =  this.supplierDao.query(queryObject);
		return _result;
	}
	
}
