package org.openxma.dsl.reference.ui.browsemodes.client;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import at.spardat.xma.component.EmbeddableComponent;
import at.spardat.xma.session.XMASessionClient;

/**
 * This class implements the generic client part of BrowseModes.
 * It provides the mapping between typeIds and classnames
 * of all pages in the component.
 *
 */
abstract public class BrowseModesGen extends EmbeddableComponent implements IBrowseModes {

    /**
     * Initializes the BrowseModesGen. This method is called
     * by the XMA-Runtime.
     *
     * @param session the XMASession, this BrowseModesGen belongs to.
     */
    public BrowseModesGen (XMASessionClient session) {
        super (session, false);
    }

    /** the unique typeId for the page BrowseModesPage */
    public static final short BrowseModesPageClass = 0;

    /**
    * Array of the classnames of all pages in
    * this component. The typeIds are used as indices
    * into this array.
     */
    static final String[] modelClasses = {
        "BrowseModesPage"
    };

    /**
    * Gets the short classname for the given typeId of
    * a page in this component. The short classname does
    * not contain the package.
    *
    * @param typeId the unique typeId
    * @return the short classname
     */
    static public String getShortModelClass(short typeId) {
        return modelClasses[typeId];
    }

    /**
    * Gets the full classname for the given typeId of
    * a page in this component.
    *
    * @param typeId the unique typeId
    * @return the full classname
     */
    public String getModelClass(short typeId) {
        return "org.openxma.dsl.reference.ui.browsemodes.client."+modelClasses[typeId];
    }

    /**
     * Invokes the BrowseModes. This method is a default
     * implementation, which invokes the first dialog page of the component.
     * Modeled init functions are called by this method.
     *
     * @param parentComposite a composite of the calling page. In most
     *     cases it will be a shell. It my be null.
     */
    public void invoke(Composite parentComposite) {
        BrowseModesPage browseModesPage_generatedXMAPage;
        if(parentComposite instanceof Shell) {
            browseModesPage_generatedXMAPage = new BrowseModesPage(this,(Shell)parentComposite);
        } else {
            browseModesPage_generatedXMAPage = new BrowseModesPage(this);
        }
        browseModesPage_generatedXMAPage.invoke();
    }

    /**
     * Gives the URI of the help for this component
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this component
     */
    public String getHelpUri() {
        return "help/org/openxma/dsl/reference/ui/browsemodes/BrowseModes.html";
    }

}
