package org.openxma.dsl.reference.ui.layouting.server;

import java.util.Locale;
import at.spardat.xma.page.*;
import at.spardat.xma.component.*;
import at.spardat.xma.mdl.*;

/**
 * This class implements the generic server part of LayoutingPage.
 * It implements the models of the page.
 *
 */
public abstract class LayoutingPageGen extends PageServer {
    AbsoluteAttachment absoluteAttachment;
    RelativeAttachment relativeAttachment;
    TabulatorAttachment tabulatorAttachment;
    AbsoluteCharAttachment absoluteCharAttachment;
    SiblingAttachment siblingAttachment;
    FieldParts fieldParts;
    WModel[] widgetModels;


    /**
     * Constructs the LayoutingPageGen.
     * This constructor is called by the runtime. Do not remove!
     *
     * @param component the component this page belongs to
     */
    public LayoutingPageGen(ComponentServer component) {
        super(component, false);
        createModels();
    }

    /**
     * Create and initialize the WidgetModels of the page.
     */
    public void createModels() {

        widgetModels = new WModel[] {  };
    }

    /*
     * returns the Locale used for this page
    */
    final protected Locale getGenPageLocale() {
        return getSession().getContext().getLocale();
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return Layouting.LayoutingPageClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public Layouting getTypedComponent() {
        return (Layouting) getComponent();
    }

    /**
     * Copies values of all attached widget models to attributes of the BusinessData instance variables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void modelToInstanceData() {
        if(absoluteAttachment!=null) {
            absoluteAttachment.modelToInstanceData();
        }
        if(relativeAttachment!=null) {
            relativeAttachment.modelToInstanceData();
        }
        if(fieldParts!=null) {
            fieldParts.modelToInstanceData();
        }
    }

    /**
     * Copies attribute values of the BusinessData instance variables into their attached widget models.
     * This includes simple widget models and tables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void instanceDataToModel() {
        if(absoluteAttachment!=null) {
            absoluteAttachment.instanceDataToModel();
        }
        if(relativeAttachment!=null) {
            relativeAttachment.instanceDataToModel();
        }
        if(fieldParts!=null) {
            fieldParts.instanceDataToModel();
        }
    }

    /**
     * Sets the given subpage. The method automatically chooses the
     * correct member variable to set.
     * @param child the subpage to set.
     */
    public void setChild(PageServer child) {
        if(child instanceof AbsoluteAttachment) {
            absoluteAttachment = (AbsoluteAttachment) child;
        } else if(child instanceof RelativeAttachment) {
            relativeAttachment = (RelativeAttachment) child;
        } else if(child instanceof TabulatorAttachment) {
            tabulatorAttachment = (TabulatorAttachment) child;
        } else if(child instanceof AbsoluteCharAttachment) {
            absoluteCharAttachment = (AbsoluteCharAttachment) child;
        } else if(child instanceof SiblingAttachment) {
            siblingAttachment = (SiblingAttachment) child;
        } else if(child instanceof FieldParts) {
            fieldParts = (FieldParts) child;
        }
    }

    /**
     * Removes the given subpage. The method automatically chooses the
     * correct member variable to clear.
     * @param child the subpage to set.
     */
    public void removeChild(PageServer child) {
        if(child instanceof AbsoluteAttachment) {
            absoluteAttachment = null;
        } else if(child instanceof RelativeAttachment) {
            relativeAttachment = null;
        } else if(child instanceof TabulatorAttachment) {
            tabulatorAttachment = null;
        } else if(child instanceof AbsoluteCharAttachment) {
            absoluteCharAttachment = null;
        } else if(child instanceof SiblingAttachment) {
            siblingAttachment = null;
        } else if(child instanceof FieldParts) {
            fieldParts = null;
        }
    }

}
