package org.openxma.dsl.reference.ui.panels.server;

import java.util.Locale;
import at.spardat.xma.page.*;
import at.spardat.xma.component.*;
import at.spardat.xma.mdl.*;
import at.spardat.enterprise.util.*;
import at.spardat.xma.mdl.simple.*;
import at.spardat.enterprise.fmt.*;

/**
 * This class implements the generic server part of HPanelPage.
 * It implements the models of the page.
 *
 */
public abstract class HPanelPageGen extends PageServer {
    ISimpleWM bigText;
    ISimpleWM tx1;
    ISimpleWM tx2;
    ISimpleWM tx3;
    ISimpleWM tx4;
    ISimpleWM tx5;
    ISimpleWM tx6;
    WModel[] widgetModels;


    /**
     * Constructs the HPanelPageGen.
     * This constructor is called by the runtime. Do not remove!
     *
     * @param component the component this page belongs to
     */
    public HPanelPageGen(ComponentServer component) {
        super(component, false);
        createModels();
    }

    /**
     * Create and initialize the WidgetModels of the page.
     */
    public void createModels() {
        IFmt formatter;
        bigText = new SimpleWM((short) 0, Types.T_STRING, this);
        formatter = AStringFmt.getInstance(-1);
        bigText.setFmt(formatter);
        tx1 = new SimpleWM((short) 1, Types.T_STRING, this);
        formatter = AStringFmt.getInstance(-1);
        tx1.setFmt(formatter);
        tx2 = new SimpleWM((short) 2, Types.T_STRING, this);
        formatter = AStringFmt.getInstance(-1);
        tx2.setFmt(formatter);
        tx3 = new SimpleWM((short) 3, Types.T_STRING, this);
        formatter = AStringFmt.getInstance(-1);
        tx3.setFmt(formatter);
        tx4 = new SimpleWM((short) 4, Types.T_STRING, this);
        formatter = AStringFmt.getInstance(-1);
        tx4.setFmt(formatter);
        tx5 = new SimpleWM((short) 5, Types.T_STRING, this);
        formatter = AStringFmt.getInstance(-1);
        tx5.setFmt(formatter);
        tx6 = new SimpleWM((short) 6, Types.T_STRING, this);
        formatter = AStringFmt.getInstance(-1);
        tx6.setFmt(formatter);

        widgetModels = new WModel[] { (WModel) bigText, (WModel) tx1, (WModel) tx2, (WModel) tx3, (WModel) tx4, (WModel) tx5, (WModel) tx6 };
    }

    /*
     * returns the Locale used for this page
    */
    final protected Locale getGenPageLocale() {
        return getSession().getContext().getLocale();
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return Panels.HPanelPageClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public Panels getTypedComponent() {
        return (Panels) getComponent();
    }

    /**
     * Sets the given subpage. The method automatically chooses the
     * correct member variable to set.
     * @param child the subpage to set.
     */
    public void setChild(PageServer child) {
       // no subpages - nothing to do
    }

    /**
     * Removes the given subpage. The method automatically chooses the
     * correct member variable to clear.
     * @param child the subpage to set.
     */
    public void removeChild(PageServer child) {
       // no subpages - nothing to do
    }

}
